<?php
/*
	Copyright (C) 2008 Giovanni Vallesi
	Copyright (C) 2008, 2009 CAST MediaTel <http://www.castmediatel.com/>.
	All rights reserved.

	part of a package for AskoziaPBX (http://askozia.com/pbx)
	Copyright (C) 2008 IKT <http://itison-ikt.de>.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
	
	1. Redistributions of source code must retain all the above copyrights notices,
	   this list of conditions and the following disclaimer.
	
	2. Redistributions in binary form must reproduce all the above copyrights
	   notices, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	
	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
	AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/

error_reporting(1024);
/* include system functions  */
require_once("config.inc");
require_once("globals.inc");
require_once("guiconfig.inc");
/* generic use */
define('PKG_NAME', 'astmedia');
define('PKG_MVSUFFIX', '_mv_' .PKG_NAME);
define('PKG_TITLE', 'Asterisk live media for AskoziaPBX');
/* paths, files, mount points*/
define('PKG_INDEXLINK', 'packages_' . PKG_NAME . '.php');
$info_pkg = packages_get_package(PKG_NAME);
define('PKG_ROOT', $info_pkg['path']);
define('PKG_MEDIAROOT', PKG_ROOT . '/sys.overlap');
define('PKG_DATAPATH', $info_pkg['datapath']);
define('PKG_LOGFILE', $info_pkg['datapath'] . '/astmedia.log');
define('PKG_SOUNDS', '/usr/local/share/asterisk/sounds');
define('PKG_MOH', '/usr/local/share/asterisk/moh');
define('PKG_VOICEMAIL', '/var/spool/asterisk/voicemail');
define('PKG_ASTDB', '/usr/local/share/asterisk/astdb');
define('PKG_AGIBIN', '/usr/local/share/asterisk/agi-bin');
define('PKG_KEYS', '/usr/local/share/asterisk/keys');
define('PKG_MODULES', '/asterisk/modules');
define('PKG_ASTMOUNTP', 'asterisk');
/* flags */

/* global variables */
$msg_buf = '';
/* cleanup */
unset($info_pkg);
?>